@@ -2,6 +2,7 @@ import asyncio
 import base64
 import getpass
 import logging
+import re
 import sys
 from functools import partial
 from typing import (
@@ -11,11 +12,13 @@ from typing import (
 )
 
 from .api import (
+    BadRequest,
     BaseObject,
     Error,
 )
 from .api.errors import (
     AioTDLibError,
+    NotFound,
     Unauthorized,
 )
 
@@ -51,6 +54,16 @@ def str_to_base64(text: Union[str, bytes]) -> str:
     return base64.b64encode(result).decode()
 
 
+def strip_phone_number_symbols(phone_number: str) -> str:
+    if not isinstance(phone_number, str):
+        try:
+            phone_number = str(phone_number)
+        except ValueError:
+            raise ValueError(f'Phone number should be an instance of str, not a {phone_number.__class__.__name__}')
+
+    return re.sub(r'(?<!^)|[^\d]+', '', phone_number)
+
+
 class PendingRequest:
     request: Optional[BaseObject] = None
     update: Optional[BaseObject] = None
@@ -83,8 +96,12 @@ class PendingRequest:
 
     def raise_error(self):
         if isinstance(self.update, Error):
-            if self.update.code == 401:
+            if self.update.code == 400:
+                raise BadRequest(self.update.message)
+            elif self.update.code == 401:
                 raise Unauthorized(self.update.message)
+            elif self.update.code == 404:
+                raise NotFound(self.update.message)
 
             raise AioTDLibError(self.update.code, self.update.message)
 
